@@ -1,19 +1,21 @@
 # encoding: utf-8
 
 from ..utils import MISSING_OBJECT
+from .annotation import AggregateProperty, AnnotationProperty, RelatedExistenceCheckProperty
 from .base import QueryableProperty, queryable_property
 from .cache_behavior import CACHE_RETURN_VALUE, CACHE_VALUE, CLEAR_CACHE, DO_NOTHING
-from .common import (AggregateProperty, AnnotationProperty, MappingProperty, RangeCheckProperty,
-                     RelatedExistenceCheckProperty, ValueCheckProperty)
 from .mixins import (AnnotationGetterMixin, AnnotationMixin, boolean_filter, LookupFilterMixin, lookup_filter,
                      REMAINING_LOOKUPS, SetterMixin, UpdateMixin)
+from .specialized import MappingProperty, RangeCheckProperty, ValueCheckProperty
+from .subquery import SubqueryExistenceCheckProperty, SubqueryFieldProperty
 
 __all__ = (
     'MISSING_OBJECT',
+    'AggregateProperty', 'AnnotationProperty', 'RelatedExistenceCheckProperty',
     'QueryableProperty', 'queryable_property',
     'CACHE_RETURN_VALUE', 'CACHE_VALUE', 'CLEAR_CACHE', 'DO_NOTHING',
-    'AggregateProperty', 'AnnotationProperty', 'MappingProperty', 'RangeCheckProperty', 'RelatedExistenceCheckProperty',
-    'ValueCheckProperty',
     'AnnotationGetterMixin', 'AnnotationMixin', 'boolean_filter', 'LookupFilterMixin', 'lookup_filter',
     'REMAINING_LOOKUPS', 'SetterMixin', 'UpdateMixin',
+    'MappingProperty', 'RangeCheckProperty', 'ValueCheckProperty',
+    'SubqueryExistenceCheckProperty', 'SubqueryFieldProperty',
 )
