@@ -1,14 +1,18 @@
 # encoding: utf-8
-import pytest
+from collections import Counter
 
+import pytest
 import six
 from django import VERSION as DJANGO_VERSION
-from django.db.models import CharField, IntegerField, Q, Sum
+from django.db.models import CharField, Count, IntegerField, Q, Sum
 from six.moves import cPickle
 
+from queryable_properties.exceptions import QueryablePropertyDoesNotExist, QueryablePropertyError
+from queryable_properties.properties.base import QueryablePropertyDescriptor
+from queryable_properties.utils import get_queryable_property
 from queryable_properties.utils.internal import (
-    get_output_field, InjectableMixin, MISSING_OBJECT, ModelAttributeGetter, parametrizable_decorator,
-    QueryablePropertyReference, QueryPath, resolve_queryable_property, TreeNodeProcessor
+    get_output_field, get_queryable_property_descriptor, InjectableMixin, MISSING_OBJECT, ModelAttributeGetter,
+    parametrizable_decorator, QueryablePropertyReference, QueryPath, resolve_queryable_property, TreeNodeProcessor
 )
 
 from ..app_management.models import (ApplicationWithClassBasedProperties, ApplicationWithDecoratorBasedProperties,
@@ -215,11 +219,39 @@ class TestModelAttributeGetter(object):
         ('attr', QueryPath('attr')),
         ('attr1.attr2', QueryPath('attr1__attr2')),
         ('attr1.attr2.attr3', QueryPath('attr1__attr2__attr3')),
+        (['attr1', 'attr2'], QueryPath('attr1__attr2')),
+        (('attr1', 'attr2', 'attr3'), QueryPath('attr1__attr2__attr3')),
     ])
     def test_initializer(self, path, expected_query_path):
         getter = ModelAttributeGetter(path)
         assert getter.query_path == expected_query_path
 
+    @pytest.mark.django_db
+    @pytest.mark.parametrize('attribute_name, expected_value', [
+        ('major', 1),
+        ('changes', None),
+    ])
+    def test_get_attribute(self, versions, attribute_name, expected_value):
+        getter = ModelAttributeGetter(())
+        assert getter._get_attribute(versions[0], attribute_name) == expected_value
+
+    def test_get_attribute_catch_attribute_error_on_none(self):
+        getter = ModelAttributeGetter(())
+        assert getter._get_attribute(None, 'non_existent') is MISSING_OBJECT
+
+    @pytest.mark.django_db
+    def test_get_attribute_bubble_attribute_error(self, versions):
+        getter = ModelAttributeGetter(())
+        with pytest.raises(AttributeError):
+            getter._get_attribute(versions[0], 'non_existent')
+
+    @pytest.mark.django_db
+    def test_get_attribute_catch_object_does_not_exist(self, applications, versions):
+        obj = versions[0].__class__.objects.get(pk=versions[0].pk)  # Refresh from DB
+        applications[0].delete()
+        getter = ModelAttributeGetter(())
+        assert getter._get_attribute(obj, 'application') is MISSING_OBJECT
+
     @pytest.mark.django_db
     @pytest.mark.parametrize('path, expected_value', [
         ('major', 1),
@@ -232,26 +264,27 @@ class TestModelAttributeGetter(object):
         assert getter.get_value(obj) == expected_value
 
     @pytest.mark.django_db
-    def test_get_value_catch_attribute_error_on_none(self, versions):
-        obj = versions[0]
-        getter = ModelAttributeGetter('changes.strip')
-        assert getter.get_value(obj) is MISSING_OBJECT
-
-    @pytest.mark.django_db
-    @pytest.mark.parametrize('path', ['non_existent', 'non.existent', 'application.non_existent'])
-    def test_get_value_bubble_attribute_error(self, versions, path):
-        obj = versions[0]
-        getter = ModelAttributeGetter(path)
-        with pytest.raises(AttributeError):
-            getter.get_value(obj)
-
-    @pytest.mark.django_db
-    def test_get_value_catch_object_does_not_exist(self, applications, versions):
+    def test_get_value_missing_object(self, applications, versions):
         obj = versions[0].__class__.objects.get(pk=versions[0].pk)  # Refresh from DB
         applications[0].delete()
         getter = ModelAttributeGetter('application.name')
         assert getter.get_value(obj) is MISSING_OBJECT
 
+    @pytest.mark.django_db
+    @pytest.mark.usefixtures('versions')
+    def test_get_values(self, categories):
+        getter = ModelAttributeGetter('applications.versions.major')
+        assert Counter(getter.get_values(categories[0])) == Counter({2: 2, 1: 6})
+
+    @pytest.mark.django_db
+    @pytest.mark.usefixtures('versions')
+    def test_get_values_missing_object(self, categories):
+        getter = ModelAttributeGetter('applications.versions.major')
+        VersionWithClassBasedProperties.objects.all().delete()
+        assert getter.get_values(categories[0]) == []
+        ApplicationWithClassBasedProperties.objects.all().delete()
+        assert getter.get_values(categories[0]) == []
+
     @pytest.mark.parametrize('path, lookup, value, expected_query_name', [
         ('attr', 'exact', 1337, 'attr__exact'),
         ('attr1.attr2', 'in', ('test', 'value'), 'attr1__attr2__in'),
@@ -288,62 +321,132 @@ def test_parametrizable_decorator():
     assert func3.kwargs == dict(kwarg='test')
 
 
+class TestGetQueryablePropertyDescriptor(object):
+
+    @pytest.mark.parametrize('model, property_name', [
+        (VersionWithClassBasedProperties, 'major_minor'),
+        (VersionWithDecoratorBasedProperties, 'major_minor'),
+        (VersionWithClassBasedProperties, 'version'),
+        (VersionWithDecoratorBasedProperties, 'version'),
+    ])
+    def test_property_found(self, model, property_name):
+        descriptor = get_queryable_property_descriptor(model, property_name)
+        assert isinstance(descriptor, QueryablePropertyDescriptor)
+
+    @pytest.mark.parametrize('model, property_name', [
+        (VersionWithClassBasedProperties, 'non_existent'),
+        (VersionWithDecoratorBasedProperties, 'non_existent'),
+        (VersionWithClassBasedProperties, 'major'),  # Existing model field
+        (VersionWithDecoratorBasedProperties, 'major'),  # Existing model field
+    ])
+    def test_exception(self, model, property_name):
+        with pytest.raises(QueryablePropertyDoesNotExist):
+            get_queryable_property_descriptor(model, property_name)
+
+
+class TestQueryablePropertyReference(object):
+
+    @pytest.mark.parametrize('relation_path, expected_result', [
+        (QueryPath(), QueryPath('dummy')),
+        (QueryPath('application'), QueryPath('application__dummy')),
+    ])
+    def test_full_path(self, relation_path, expected_result):
+        prop = get_queryable_property(ApplicationWithClassBasedProperties, 'dummy')
+        ref = QueryablePropertyReference(prop, prop.model, relation_path)
+        assert ref.full_path == expected_result
+
+    def test_descriptor(self):
+        prop = get_queryable_property(ApplicationWithClassBasedProperties, 'dummy')
+        ref = QueryablePropertyReference(prop, prop.model, QueryPath())
+        assert ref.descriptor == ApplicationWithClassBasedProperties.dummy
+
+    @pytest.mark.parametrize('lookups, relation_path, expected_filter', [
+        (QueryPath(), QueryPath(), 'version_count__exact'),
+        (QueryPath('lt'), QueryPath(), 'version_count__lt'),
+        (QueryPath('date__year'), QueryPath('application'), 'application__version_count__date__year'),
+    ])
+    def test_get_filter(self, lookups, relation_path, expected_filter):
+        prop = get_queryable_property(ApplicationWithClassBasedProperties, 'version_count')
+        ref = QueryablePropertyReference(prop, prop.model, relation_path)
+        q = ref.get_filter(lookups, 1337)
+        assert isinstance(q, Q)
+        assert q.children == [(expected_filter, 1337)]
+
+    def test_get_filter_exception(self):
+        prop = get_queryable_property(ApplicationWithClassBasedProperties, 'dummy')
+        ref = QueryablePropertyReference(prop, prop.model, QueryPath())
+        with pytest.raises(QueryablePropertyError):
+            ref.get_filter(QueryPath(), None)
+
+    def test_get_annotation(self):
+        prop = get_queryable_property(ApplicationWithClassBasedProperties, 'version_count')
+        ref = QueryablePropertyReference(prop, prop.model, QueryPath())
+        assert isinstance(ref.get_annotation(), Count)
+
+    def test_get_annotation_exception(self):
+        prop = get_queryable_property(ApplicationWithClassBasedProperties, 'dummy')
+        ref = QueryablePropertyReference(prop, prop.model, QueryPath())
+        with pytest.raises(QueryablePropertyError):
+            ref.get_annotation()
+
+
 class TestResolveQueryableProperty(object):
 
     @pytest.mark.parametrize('model, query_path, expected_property, expected_lookups', [
         # No relation involved
-        (VersionWithClassBasedProperties, QueryPath('version'), VersionWithClassBasedProperties.version, QueryPath()),
+        (VersionWithClassBasedProperties, QueryPath('version'),
+         get_queryable_property(VersionWithClassBasedProperties, 'version'), QueryPath()),
         (VersionWithDecoratorBasedProperties, QueryPath('version'),
-         VersionWithDecoratorBasedProperties.version, QueryPath()),
+         get_queryable_property(VersionWithDecoratorBasedProperties, 'version'), QueryPath()),
         (VersionWithClassBasedProperties, QueryPath('version__lower__exact'),
-         VersionWithClassBasedProperties.version, QueryPath('lower__exact')),
+         get_queryable_property(VersionWithClassBasedProperties, 'version'), QueryPath('lower__exact')),
         (VersionWithDecoratorBasedProperties, QueryPath('version__lower__exact'),
-         VersionWithDecoratorBasedProperties.version, QueryPath('lower__exact')),
+         get_queryable_property(VersionWithDecoratorBasedProperties, 'version'), QueryPath('lower__exact')),
         # FK forward relation
         (VersionWithClassBasedProperties, QueryPath('application__version_count'),
-         ApplicationWithClassBasedProperties.version_count, QueryPath()),
+         get_queryable_property(ApplicationWithClassBasedProperties, 'version_count'), QueryPath()),
         (VersionWithDecoratorBasedProperties, QueryPath('application__version_count'),
-         ApplicationWithDecoratorBasedProperties.version_count, QueryPath()),
+         get_queryable_property(ApplicationWithDecoratorBasedProperties, 'version_count'), QueryPath()),
         (VersionWithClassBasedProperties, QueryPath('application__major_sum__gt'),
-         ApplicationWithClassBasedProperties.major_sum, QueryPath('gt')),
+         get_queryable_property(ApplicationWithClassBasedProperties, 'major_sum'), QueryPath('gt')),
         (VersionWithDecoratorBasedProperties, QueryPath('application__major_sum__gt'),
-         ApplicationWithDecoratorBasedProperties.major_sum, QueryPath('gt')),
+         get_queryable_property(ApplicationWithDecoratorBasedProperties, 'major_sum'), QueryPath('gt')),
         # FK reverse relation
         (ApplicationWithClassBasedProperties, QueryPath('versions__major_minor'),
-         VersionWithClassBasedProperties.major_minor, QueryPath()),
+         get_queryable_property(VersionWithClassBasedProperties, 'major_minor'), QueryPath()),
         (ApplicationWithDecoratorBasedProperties, QueryPath('versions__major_minor'),
-         VersionWithDecoratorBasedProperties.major_minor, QueryPath()),
+         get_queryable_property(VersionWithDecoratorBasedProperties, 'major_minor'), QueryPath()),
         (ApplicationWithClassBasedProperties, QueryPath('versions__version__lower__contains'),
-         VersionWithClassBasedProperties.version, QueryPath('lower__contains')),
+         get_queryable_property(VersionWithClassBasedProperties, 'version'), QueryPath('lower__contains')),
         (ApplicationWithDecoratorBasedProperties, QueryPath('versions__version__lower__contains'),
-         VersionWithDecoratorBasedProperties.version, QueryPath('lower__contains')),
+         get_queryable_property(VersionWithDecoratorBasedProperties, 'version'), QueryPath('lower__contains')),
         # M2M forward relation
         (ApplicationWithClassBasedProperties, QueryPath('categories__circular'),
-         CategoryWithClassBasedProperties.circular, QueryPath()),
+         get_queryable_property(CategoryWithClassBasedProperties, 'circular'), QueryPath()),
         (ApplicationWithDecoratorBasedProperties, QueryPath('categories__circular'),
-         CategoryWithDecoratorBasedProperties.circular, QueryPath()),
+         get_queryable_property(CategoryWithDecoratorBasedProperties, 'circular'), QueryPath()),
         (ApplicationWithClassBasedProperties, QueryPath('categories__circular__exact'),
-         CategoryWithClassBasedProperties.circular, QueryPath('exact')),
+         get_queryable_property(CategoryWithClassBasedProperties, 'circular'), QueryPath('exact')),
         (ApplicationWithDecoratorBasedProperties, QueryPath('categories__circular__exact'),
-         CategoryWithDecoratorBasedProperties.circular, QueryPath('exact')),
+         get_queryable_property(CategoryWithDecoratorBasedProperties, 'circular'), QueryPath('exact')),
         # M2M reverse relation
         (CategoryWithClassBasedProperties, QueryPath('applications__major_sum'),
-         ApplicationWithClassBasedProperties.major_sum, QueryPath()),
+         get_queryable_property(ApplicationWithClassBasedProperties, 'major_sum'), QueryPath()),
         (CategoryWithDecoratorBasedProperties, QueryPath('applications__major_sum'),
-         ApplicationWithDecoratorBasedProperties.major_sum, QueryPath()),
+         get_queryable_property(ApplicationWithDecoratorBasedProperties, 'major_sum'), QueryPath()),
         (CategoryWithClassBasedProperties, QueryPath('applications__version_count__lt'),
-         ApplicationWithClassBasedProperties.version_count, QueryPath('lt')),
+         get_queryable_property(ApplicationWithClassBasedProperties, 'version_count'), QueryPath('lt')),
         (CategoryWithDecoratorBasedProperties, QueryPath('applications__version_count__lt'),
-         ApplicationWithDecoratorBasedProperties.version_count, QueryPath('lt')),
+         get_queryable_property(ApplicationWithDecoratorBasedProperties, 'version_count'), QueryPath('lt')),
         # Multiple relations
         (CategoryWithClassBasedProperties, QueryPath('applications__versions__application__categories__circular'),
-         CategoryWithClassBasedProperties.circular, QueryPath()),
+         get_queryable_property(CategoryWithClassBasedProperties, 'circular'), QueryPath()),
         (CategoryWithDecoratorBasedProperties, QueryPath('applications__versions__application__categories__circular'),
-         CategoryWithDecoratorBasedProperties.circular, QueryPath()),
+         get_queryable_property(CategoryWithDecoratorBasedProperties, 'circular'), QueryPath()),
         (VersionWithClassBasedProperties, QueryPath('application__categories__circular__in'),
-         CategoryWithClassBasedProperties.circular, QueryPath('in')),
+         get_queryable_property(CategoryWithClassBasedProperties, 'circular'), QueryPath('in')),
         (VersionWithDecoratorBasedProperties, QueryPath('application__categories__circular__in'),
-         CategoryWithDecoratorBasedProperties.circular, QueryPath('in')),
+         get_queryable_property(CategoryWithDecoratorBasedProperties, 'circular'), QueryPath('in')),
     ])
     def test_successful(self, model, query_path, expected_property, expected_lookups):
         expected_ref = QueryablePropertyReference(expected_property, expected_property.model,
